@@ -11,6 +11,9 @@ use them as well, so they are kept here.
 import re
 import os.path
 import sys
+# site
+# WARNING: since this is imported by setup.py, it's critical the toplevel
+#          not depend on any imports outside of stdlib & setuptools
 # local
 __all__ = [
     # constants
@@ -29,7 +32,7 @@ __all__ = [
 #=============================================================================
 # constants
 #=============================================================================
-__version__ = "1.8.0"
+__version__ = '1.9.4'
 
 # names of standard cloud extensions used by most cloud themes
 std_exts = [
@@ -38,7 +41,7 @@ std_exts = [
     'cloud_sptheme.ext.docfield_markup',
     'cloud_sptheme.ext.escaped_samp_literals',
     'cloud_sptheme.ext.index_styling',
-    'cloud_sptheme.ext.relbar_toc',
+    'cloud_sptheme.ext.relbar_links',
     'cloud_sptheme.ext.table_styling',
 ]
 
