@@ -10,15 +10,20 @@ else:
     # FIXME: not present in py25
     from itertools import izip_longest
 import os
-from shutil import copyfile
 # site
 from docutils import nodes
 from docutils.parsers.rst import directives
 from docutils.parsers.rst.directives.tables import RSTTable
 from sphinx.builders.html import StandaloneHTMLBuilder
+try:
+    # sphinx 1.5 has it's own RSTTable, inherit from that if available.
+    from sphinx.directives.patches import RSTTable
+except ImportError:
+    pass
 # pkg
 from cloud_sptheme import __version__
 from cloud_sptheme import _root_dir, is_cloud_theme
+from cloud_sptheme.utils import add_static_file
 # local
 __all__ = [
     "setup",
@@ -103,6 +108,7 @@ def divider_list(argument):
 #=============================================================================
 # replacement for table directive
 #=============================================================================
+
 class ExtendedRSTTable(RSTTable):
     # TODO: could have this auto-generate tabularcolumns directive for latex,
     #       based on alignment, dividers, and wrapping settings:
@@ -114,12 +120,12 @@ class ExtendedRSTTable(RSTTable):
     option_spec = RSTTable.option_spec.copy()
     option_spec.update({
         # class, name already present
-        ##'header-rows': directives.nonnegative_int,
         'header-columns': directives.nonnegative_int,
         # TODO: column-widths: support limited set of units (em/in/%)
         #       expressable under both css & latex
         'widths': directives.positive_int_list,
         'column-alignment': alignment_list,
+        'header-alignment': alignment_list,
         'column-wrapping': bool_list,
         'column-classes': class_option_list,
         'column-dividers': divider_list,
@@ -132,30 +138,33 @@ class ExtendedRSTTable(RSTTable):
         return result
 
     def _update_table_classes(self, table):
+        # add global "table_styling_class"
         assert isinstance(table, nodes.table)
         config = self.state.document.settings.env.config
         classes = getattr(config, CLASS_KEY)
         if classes:
             for cls in classes.split():
                 table['classes'].append(cls)
-        header_cols = self.options.get("header-columns") or 0
-##        header_rows = self.options.get("header-rows")
-        widths = self.options.get("widths")
-        dividers = self.options.get("column-dividers")
-        if dividers is None:
-            get_divider = None
-        else:
-            def get_divider(idx):
-                try:
-                    return dividers[idx]
-                except IndexError:
-                    return "no"
+
+        # figure out how many header rows & columns
+        options = self.options
+        header_cols = options.get("header-columns") or 0
+        widths = options.get("widths")
+
+        # Parse column options into list of ColumnOptions records.
+        # If any option is short, it will be padded with ``None`` values
+        # to match the longest list.
         EMPTY = ()
-        opts = (
-            self.options.get("column-alignment", EMPTY),
-            self.options.get("column-wrapping", EMPTY),
-            self.options.get("column-classes", EMPTY),
-        )
+        opts = tuple(izip_longest(
+            options.get("column-alignment", EMPTY),
+            options.get("header-alignment", EMPTY),
+            options.get("column-wrapping", EMPTY),
+            options.get("column-classes", EMPTY),
+            options.get("column-dividers", EMPTY),
+        ))
+        NULL_OPTIONS = [None] * 5
+
+        # try to find "tgroup" node holding thead/tbody etc
         def locate(cls):
             for child in table.children:
                 if isinstance(child, cls):
@@ -164,49 +173,93 @@ class ExtendedRSTTable(RSTTable):
         tgroup = locate(nodes.tgroup)
         if not tgroup:
             return
-        col = 0
+
+        # loop through content of 'tgroup' node --
+        # should be 0+ 'colspec' nodes, 0-1 thead node, and 0-1 tbody nodes.
+        # NOTE: using 'cgroup_idx' as hack to track colgroup index
+        cgroup_idx = 0
         for child in tgroup:
+            # handle colspec entry (corresponding to COLGROUP)
+            # set group width & mark "header" columns
             if isinstance(child, nodes.colspec):
-                if widths and col < len(widths):
-                    child['colwidth'] = widths[col]
-                if col < header_cols:
+                if widths and cgroup_idx < len(widths):
+                    child['colwidth'] = widths[cgroup_idx]
+                if cgroup_idx < header_cols:
                     child['stub'] = 1
-                col += 1
+                cgroup_idx += 1
                 continue
+
+            # otherwise have thead / tbody --
+            # loop through each row in child, adding classes as appropriate
             assert isinstance(child, (nodes.thead, nodes.tbody))
             for row in child.children:
-                # add alignment and wrap classes to each entry (would add to
-                # colspec, but html doesn't inherit much from colgroup)
+
+                # check if we're in a header
+                is_header_row = isinstance(child, nodes.thead)
+
+                # Add alignment and wrap classes to each column (would add to
+                # colspec instead, but html doesn't inherit much from colgroup)
+                # We iterate trough the list of table cells in the row while
+                # maintaining the column format index in parallel
                 assert isinstance(row, nodes.row)
-                for idx, (entry, align, wrap, clist) in \
-                  enumerate(izip_longest(row, *opts)):
-                    if entry is None:
-                        # FIXME: make into propert rst error
-                        raise ValueError("not enough columns for field options")
-                    assert isinstance(entry, nodes.entry)
-                    classes = entry['classes']
+                colidx = 0  # visible idx, to account for colspans
+                prev_col_classes = None
+                for idx, col in enumerate(row):
+
+                    # get align/wrap options for column
+                    try:
+                        col_opts = opts[colidx]
+                    except IndexError:
+                        col_opts = NULL_OPTIONS
+                    align, header_align, wrap, clist, divider = col_opts
+
+                    # let header values override defaults
+                    if is_header_row:
+                        align = header_align
+                    if align is None:
+                        align = "left"
+
+                    # add alignment / wrapping classes for column
+                    assert isinstance(col, nodes.entry)
+                    classes = col['classes']
                     if align:
                         classes.append(align + "-align")
                     if wrap is False:
                         classes.append("nowrap")
                     if clist:
                         classes.extend(clist)
-                    if get_divider:
-                        classes.append(get_divider(idx) + "-left-divider")
-                        classes.append(get_divider(idx+1) + "-right-divider")
-        # untested - might be missing some docutils node framework bits
-        ##if header_rows > 1:
-        ##    thead = locate(nodes.thead)
-        ##    if not thead:
-        ##        thead = nodes.thead()
-        ##        idx = table.children.index(tgroup)
-        ##        table.children.insert(idx, thead)
-        ##    thead.extend(tgroup.children[:header_rows])
-        ##    del tgroup.children[:header_rows]
+
+                    if divider:
+                        # add divider class for left side of this column,
+                        classes.append(divider + "-left-divider")
+
+                        # same divider type to right side of previous column,
+                        # so that css can bind to either.
+                        if prev_col_classes is not None:
+                            prev_col_classes.append(divider + "-right-divider")
+
+                    # update prev_col_classes for next pass
+                    prev_col_classes = classes
+
+                    # Now it's the time to advance the column format index.
+                    # If we have a cell spans across the columns, we add the
+                    # value of 'morecols', so we get the proper right divider
+                    # for the current cell and the proper format index for the
+                    # next loop iteration
+                    colidx += 1 + col.get("morecols", 0)
+
+                # add divider to right side of last column
+                if prev_col_classes and colidx < len(opts):
+                    align, header_align, wrap, clist, divider = opts[colidx]
+                    if divider:
+                        prev_col_classes.append(divider + "-right-divider")
 
 #=============================================================================
 # patch builder to copy css file (if needed)
 #=============================================================================
+
+css_path = os.path.join(_root_dir, "ext", "static", "table_styling.css")
+
 def prepare_builder(app):
     # make sure needed css styling gets included when building html
     builder = app.builder
@@ -219,17 +272,7 @@ def prepare_builder(app):
         return
 
     # add custom css stylesheet
-    name = "table_styling.css"
-    app.add_stylesheet(name)
-
-    # monkeypatch builder to copy over css file
-    orig = builder.copy_static_files
-    def wrapper():
-        orig()
-        source = os.path.join(_root_dir, "ext", name)
-        target = os.path.join(builder.outdir, "_static", name)
-        copyfile(source, target)
-    builder.copy_static_files = wrapper
+    add_static_file(builder, css_path, stylesheet=True)
 
 #=============================================================================
 # register extension
@@ -241,6 +284,13 @@ def setup(app):
     # default class to add to all styled tables
     app.add_config_value(CLASS_KEY, "styled-table", "html")
 
+    # HACK: since sphinx 1.5, issues warning about replacing RSTTable.
+    #       we know that's happening, so pop it out of registry first.
+    #       (but ONLY what's there wasn't already patched)
+    directive_map = directives._directives
+    if directive_map.get("table") is RSTTable:
+        directive_map.pop("table")
+
     # replace existing table directive with custom one
     app.add_directive("table", ExtendedRSTTable)
 
